/* ************************************************************************ */
/* main.c                                                                   */
/*                                                                          */
/* Restore A100 ARW files broken by IDC.                                    */
/*                                                                          */
/* Copyright 2010 Ronan Demoment.                                           */
/* ************************************************************************ */

/* ************************************************************************ */
/* Includes *************************************************************** */
/* ************************************************************************ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "arwparse.h"

/* -------------------------------------------------------------------------
 * Main function of the program. Parses command line & process the file
 *
 * argc     number of parameters on command line
 * argv     array of parameters
 *
 * Returns  O if successful
 * ------------------------------------------------------------------------- */
int main(int argc, char **argv)
{
    arwfile_descriptor desc;
    tag_descriptor    *model_tag;
    tag_descriptor    *width_tag;
    tag_descriptor    *height_tag;
    tag_descriptor    *bpp_tag;
    tag_descriptor    *offset_tag;
    int                i,j;
    unsigned short    *pixel, new_pixel;
    FILE              *restored_raw;
    char              restored_filename[512];

    if (argc < 2)
    {
        printf("Usage %s <filename.arw> [<filename.arw> ...]\n", argv[0]);
        exit(1);
    }

    for (j = 1; j < argc; j++)
    {
        /* Load ARW file into memory */
        if (load_arw(argv[j], &desc))
        {
            printf("%s : Can't read input file, skipping\n", argv[j]);
            continue;
        }

        /* Parse ARW file */
        parse_arw(&desc);
        //print_arw_info(&desc);


        /* Check that we have a subIFD in the first IFD */
        if (desc.ifd[0].subifd == NULL)
        {
            printf("%s : ARW file format not handled by this onverter.\n",
                   argv[j]);
            continue;
        }

        /* Retreives basic informtion on the picture */
        model_tag = get_tag(&desc.ifd[0], 0x110);
        width_tag = get_tag(desc.ifd[0].subifd, 0x100);
        height_tag = get_tag(desc.ifd[0].subifd, 0x101);
        bpp_tag = get_tag(desc.ifd[0].subifd, 0x102);
        offset_tag = get_tag(desc.ifd[0].subifd, 0x111);

        /* Check that this file is what we expect : A100 RAW file modified by IDC */
        if ( (model_tag != NULL) &&
             (width_tag != NULL) &&
             (height_tag != NULL) &&
             (bpp_tag != NULL) &&
             (offset_tag != NULL) &&
             !strncmp((const char *)(desc.data + model_tag->value), "DSLR-A100", 9) &&
             (width_tag->value == 3888) &&
             (height_tag->value == 2608) &&
             (bpp_tag->value == 14) )
        {
            /* Decrease bit per pixel value */
            *((unsigned short *)(bpp_tag->pointer)) = 12;

            /* Strip extra bits for each pixel */
            pixel = (unsigned short *) (desc.data + offset_tag->value);
            for (i = 0; i < 3888 * 2608; i++)
            {
                new_pixel = (pixel[i] >> 8) | (pixel[i] << 8);
                new_pixel >>= 2;
                pixel[i] = (new_pixel >> 8) | (new_pixel << 8);
            }

            /* Build destination filename */
            restored_filename[sizeof(restored_filename) - sizeof("_restored") - 1] = 0;
            strncpy(restored_filename, argv[j], sizeof(restored_filename) - sizeof("_restored"));
            *strrchr(restored_filename, '.') = 0;
            strcat(restored_filename, "_restored.arw");


            restored_raw = fopen(restored_filename, "wb");
            if (restored_raw != NULL)
            {
                if (fwrite(desc.data, 1, desc.filesize, restored_raw) != desc.filesize)
                {
                    printf("%s : ERROR writing output file\n", argv[j]);
                }
                fclose(restored_raw);
            }
            else
            {
                printf("%s : ERROR opening output file\n", argv[j]);
            }


            printf("%s : file restored.\n", argv[j]);
        }

        /* Free memory */
        purge_arw(&desc);
    }

    return(0);
}
